@@ -3,7 +3,7 @@ BEGIN {
   $Promises::Deferred::AUTHORITY = 'cpan:STEVAN';
 }
 {
-  $Promises::Deferred::VERSION = '0.05';
+  $Promises::Deferred::VERSION = '0.06';
 }
 # ABSTRACT: An implementation of Promises in Perl
 
@@ -52,7 +52,6 @@ sub resolve {
     $self->{'result'} = $result;
     $self->{'status'} = RESOLVING;
     $self->_notify( $self->{'resolved'}, $result );
-    $self->{'resolved'} = [];
     $self->{'status'}   = RESOLVED;
     $self;
 }
@@ -63,7 +62,6 @@ sub reject {
     $self->{'result'} = $result;
     $self->{'status'} = REJECTING;
     $self->_notify( $self->{'rejected'}, $result );
-    $self->{'rejected'} = [];
     $self->{'status'}   = REJECTED;
     $self;
 }
@@ -91,14 +89,40 @@ sub then {
     if ( $self->status eq RESOLVED ) {
         $self->resolve( @{ $self->result } );
     }
-
-    if ( $self->status eq REJECTED ) {
+    elsif ( $self->status eq REJECTED ) {
         $self->reject( @{ $self->result } );
     }
 
     $d->promise;
 }
 
+sub finalize {
+    my ($self, $callback, $error) = @_;
+
+    (ref $callback && reftype $callback eq 'CODE')
+        || confess "You must pass in a success callback";
+
+    (ref $error && reftype $error eq 'CODE')
+        || confess "You must pass in a error callback"
+            if $error;
+
+    # if we don't get an error
+    # handler, we need to chain
+    # it automatically
+    $error ||= sub { @_ };
+
+    push @{ $self->{'resolved'} } => $callback;
+    push @{ $self->{'rejected'} } => $error;
+
+    if ( $self->status eq RESOLVED ) {
+        $self->resolve( @{ $self->result } );
+    }
+    elsif ( $self->status eq REJECTED ) {
+        $self->reject( @{ $self->result } );
+    }
+    ();
+}
+
 sub _wrap {
     my ($self, $d, $f, $method) = @_;
     return sub {
@@ -121,6 +145,9 @@ sub _wrap {
 sub _notify {
     my ($self, $callbacks, $result) = @_;
     $_->( @$result ) foreach @$callbacks;
+    $self->{'resolved'} = [];
+    $self->{'rejected'} = [];
+
 }
 
 1;
@@ -135,7 +162,7 @@ Promises::Deferred - An implementation of Promises in Perl
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
@@ -213,6 +240,20 @@ the error to the next link in the chain. This allows
 error handling to be consolidated at the point in the
 chain where it makes the most sense.
 
+=item C<finalize( $callback, ?$error )>
+
+This method is used to register two callbacks, the first
+C<$callback> will be called on success and it will be
+passed all the values that were sent to the corresponding
+call to C<resolve>. The second, C<$error> is optional and
+will be called on error, and will be passed the all the
+values that were sent to the corresponding C<reject>.
+
+Unlike the C<then()> method, C<finalize()> returns an
+empty list specifically to break the chain and to avoid
+deep recursion.  See the explanation in
+L<Promises::Cookbook::Recursion>.
+
 =item C<resolve( @args )>
 
 This is the method to call upon the successful completion
